#ifndef CHUNK_H_INCLUDED
#define CHUNK_H_INCLUDED

#include <GL/gl.h>

#include "AABB.h"
#include "Level.h"
#include "Tesselator.h"
#include "Textures.h"
#include "Tile.h"

class Chunk {

	private :
    /**
     * The game level
     */
    Level *level;
    bool dirty;
    static bool initDone;

    static int Dirt;
    static int Cobblestone;
    static Tessellator TESSELLATOR;

  public :
    /**
     * Function initializing the textures used by the class
     */
    static void init() {
        if(!initDone) {
            Dirt = Textures::loadTexture("dirt.bmp", GL_NEAREST);
            Cobblestone = Textures::loadTexture("cobblestone.bmp", GL_NEAREST);
            initDone = true;
        }
    }

    /**
     * Global rebuild statistic
     */
    static int rebuiltThisFrame;
    static int updates;


  private :
    int minX, minY, minZ;
    int maxX, maxY, maxZ;

  public :
    /**
     * Bounding box values
     */
    AABB boundingBox;

  private :

    /**
     * Rendering states
     */
    GLuint lists;

  public :
    /**
     * Chunk containing a part of the tiles in a level
     *
     * @param level The game level
     * @param minX  Minimal location X
     * @param minY  Minimal location Y
     * @param minZ  Minimal location Z
     * @param maxX  Maximal location X
     * @param maxY  Maximal location Y
     * @param maxZ  Maximal location Z
     */
    Chunk(Level &level, int minX, int minY, int minZ, int maxX, int maxY, int maxZ) : level(&level), dirty(true), minX(minX), minY(minY),
                                                                                    minZ(minZ), maxX(maxX), maxY(maxY), maxZ(maxZ),
                                                                                    boundingBox(minX, minY, minZ, maxX, maxY, maxZ), lists(glGenLists(2)){
        init();
    }

    void rebuild(int layer);

    /**
     * Render all tiles in this chunk
     *
     * @param layer The render layer (Shadow layer)
     */
    void render(int layer) {
        // Rebuild chunk if dirty
        if (dirty) {
            rebuild(0);
            rebuild(1);
        }

        // Call lists id to render the chunk
        glCallList(lists + layer);
    }

    /**
     * Mark chunk as dirty. The chunk will rebuild in the next frame
     */
    void setDirty() {
        dirty = true;
    }
};


#endif // CHUNK_H_INCLUDED
