#include "Level.h"

    /**
     * Calculate light depth of given area
     *
     * @param minX Minimum on X axis
     * @param minZ Minimum on Z axis
     * @param maxX Maximum on X axis
     * @param maxZ Maximum on Z axis
     */
void Level::calcLightDepths(int minX, int minZ, int maxX, int maxZ) {
    // For each x/z position in level
    for (int x = minX; x < minX + maxX; x++) {
        for (int z = minZ; z < minZ + maxZ; z++) {

            // Get previous light depth value
            int prevDepth = lightDepths[x + z * width];

            // Calculate new light depth
            int depth = this->depth - 1;
            while (depth > 0 && !isLightBlocker(x, depth, z)) {
                depth--;
            }

            // Set new light depth
            lightDepths[x + z * width] = depth;

            // On light depth change
            if (prevDepth != depth) {
                // Get changed range
                int minTileChangeY = min(prevDepth, depth);
                int maxTileChangeY = max(prevDepth, depth);

                // Notify tile column changed
                for (list<LevelListener*>::iterator i = levelListeners.begin(); i != levelListeners.end(); i++) {
                    (**i).lightColumnChanged(x, z, minTileChangeY, maxTileChangeY);
                }
            }
        }
    }
}

    /**
     * Get bounding box of all tiles surrounded by the given bounding box
     *
     * @param boundingBox Target bounding box located in the level
     * @return List of bounding boxes representing the tiles around the given bounding box
     */
list<AABB> Level::getCubes(AABB boundingBox) {
    list<AABB> boundingBoxList;

    int minX = (int) (floor(boundingBox.minX) - 1);
    int maxX = (int) (ceil(boundingBox.maxX) + 1);
    int minY = (int) (floor(boundingBox.minY) - 1);
    int maxY = (int) (ceil(boundingBox.maxY) + 1);
    int minZ = (int) (floor(boundingBox.minZ) - 1);
    int maxZ = (int) (ceil(boundingBox.maxZ) + 1);

    // Minimum level position
    minX = max(0, minX);
    minY = max(0, minY);
    minZ = max(0, minZ);

    // Maximum level position
    maxX = min(width, maxX);
    maxY = min(depth, maxY);
    maxZ = min(height, maxZ);

    // Include all surrounding tiles
    for (int x = minX; x < maxX; x++) {
        for (int y = minY; y < maxY; y++) {
            for (int z = minZ; z < maxZ; z++) {
                if (isSolidTile(x, y, z)) {
                    boundingBoxList.push_back(AABB(x, y, z, x + 1, y + 1, z + 1));
                }
            }
        }
    }
    return boundingBoxList;
}
