#ifndef LEVELRENDERER_H_INCLUDED
#define LEVELRENDERER_H_INCLUDED

#include "Tesselator.h"
#include "Chunk.h"
#include "Level.h"
#include "HitResult.h"
#include "Player.h"
#include "Frustum.h"

class LevelRenderer : public LevelListener {

    static const int CHUNK_SIZE;

    Tessellator tessellator;
    Level *level;
    Chunk **chunks;

    int chunkAmountX;
    int chunkAmountY;
    int chunkAmountZ;

    LevelRenderer(LevelRenderer &a) : tessellator(), level(a.level), chunks(nullptr), chunkAmountX(0),
                                    chunkAmountY(0), chunkAmountZ(0) {}

    LevelRenderer &operator=(const LevelRenderer&a){(void)a;return *this;}

  public :

    LevelRenderer(Level *level);

    void setDirty(int minX, int minY, int minZ, int maxX, int maxY, int maxZ);

    /**
     * Render all chunks of the level
     *
     * @param layer The render layer
     */
    void render(int layer) {
        //printf("Render !");
        // Get current camera frustum
        //Frustum frustum = Frustum::getFrustum();

        // Reset global chunk rebuild stats
        Chunk::rebuiltThisFrame = 0;

        // For all chunks
        for(int i=0; i<chunkAmountX * chunkAmountY * chunkAmountZ; i++)
            // Render if bounding box of chunk is in frustum
            /*if (frustum.cubeInFrustum(chunks[i]->boundingBox)) {
                // Render chunk
                chunks[i]->render(layer);
            }*/
            chunks[i]->render(layer);
        }

    void pick(Player &player);

    /**
     * Render hit face of the result
     *
     * @param hitResult The hit result to render
     */
    void renderHit(HitResult &hitResult) {
        // Setup blending and color
        glEnable(GL_BLEND);
        glBlendFunc(GL_SRC_ALPHA, GL_CURRENT_BIT);
        glColor4d(1., 1., 1., .5);

        // Render face
        tessellator.init();
        Tile::rock.renderFace(tessellator, hitResult.x, hitResult.y, hitResult.z, hitResult.face);
        tessellator.flush();

        // Disable blending
        glDisable(GL_BLEND);
    }

    void lightColumnChanged(int x, int z, int minY, int maxY) {
        setDirty(x - 1, minY - 1, z - 1, x + 1, maxY + 1, z + 1);
    }

    void tileChanged(int x, int y, int z) {
        setDirty(x - 1, y - 1, z - 1, x + 1, y + 1, z + 1);
    }

    void allChanged() {
        setDirty(0, 0, 0, level->width, level->depth, level->height);
    }
};


#endif // LEVELRENDERER_H_INCLUDED
