#include "Player.h"

void Player::tick() {
    // Store previous position
    prevX = x;
    prevY = y;
    prevZ = z;

    double forward = 0.;
    double vertical = 0.;

    // Reset the position of the player
    if (keyboard.isPressed(19)) { // R
        resetPosition();
    }

    // Player movement
    if (keyboard.isPressed(200) || keyboard.isPressed(17)) { // Up, W
        forward--;
    }
    if (keyboard.isPressed(208) || keyboard.isPressed(31)) { // Down, S
        forward++;
    }
    if (keyboard.isPressed(203) || keyboard.isPressed(30)) { // Left, A
        vertical--;
    }
    if (keyboard.isPressed(205) || keyboard.isPressed(32)) {  // Right, D
        vertical++;
    }
    if ((keyboard.isPressed(57) || keyboard.isPressed(219))) { // Space, Windows Key
        if (onGround) {
            motionY = .12;
        }
    }

    // Add motion to the player using keyboard input
    moveRelative(vertical, forward, onGround ? .02 : .005);

    // Apply gravity motion
    motionY -= .005;

    // Move the player using the motion
    move(motionX, motionY, motionZ);

    // Decrease motion
    motionX *= .91;
    motionY *= .98;
    motionZ *= .91;

    // Decrease motion on ground
    if (onGround) {
        motionX *= .8;
        motionZ *= .8;
    }
}

    /**
     * Move player relative in level with collision check
     *
     * @param x Relative x
     * @param y Relative y
     * @param z Relative z
     */
void Player::move(double x, double y, double z) {
    double prevX = x;
    double prevY = y;
    double prevZ = z;

    // Get surrounded tiles
    list<AABB> aABBs = level->getCubes(boundingBox.expand(x, y, z));

    // Check for Y collision
    for(list<AABB>::iterator i = aABBs.begin(); i!= aABBs.end(); i++) {
        y = i->clipYCollide(boundingBox, y);
    }
    boundingBox.move(0., y, 0.);

    // Check for X collision
    for(list<AABB>::iterator i = aABBs.begin(); i!= aABBs.end(); i++) {
        x = i->clipXCollide(boundingBox, x);
    }
    boundingBox.move(x, 0., 0.);

    // Check for Z collision
    for(list<AABB>::iterator i = aABBs.begin(); i!= aABBs.end(); i++) {
        z = i->clipZCollide(boundingBox, z);
    }
    boundingBox.move(0., 0., z);

    // Update on ground state
    onGround = prevY != y && prevY < 0.;

    // Stop motion on collision
    if (prevX != x) motionX = 0.;
    if (prevY != y) motionY = 0.;
    if (prevZ != z) motionZ = 0.;

    // Move the actual player position
    x = (boundingBox.minX + boundingBox.maxX) / 2.;
    y = boundingBox.minY + 1.62;
    z = (boundingBox.minZ + boundingBox.maxZ) / 2.;
}
