#include "Tesselator.h"

/**
     * Add a vertex point to buffer
     *
     * @param x Vertex point x
     * @param y Vertex point y
     * @param z Vertex point z
     */
void Tessellator::vertex(float x, float y, float z) {
    // Vertex
    //printf("Vertex !\n");
    vertexBuffer.push_back(z);
    vertexBuffer.push_back(y);
    vertexBuffer.push_back(x);

    // Texture coordinate
    if (hasTexture) {
        textureCoordinateBuffer.push_back(textureV);
        textureCoordinateBuffer.push_back(textureU);
    }

    // Color coordinate
    if (hasColor) {
        colorBuffer.push_back(red);
        colorBuffer.push_back(green);
        colorBuffer.push_back(blue);
    }

    vertices++;

    // Flush if there are too many vertices in the buffer
    if (vertices == 100000) {
        flush();
    }
}

    /**
     * Render the buffer
     */
void Tessellator::flush() {
    // Maybe not necessary (We'll see that anyways)
    //vertexBuffer.flip();
    //textureCoordinateBuffer.flip();

    // Set points
    //printf("Flush !\n");
    glVertexPointer(3, GL_FLOAT, 0, vertexBuffer.data());
    if (hasTexture) {
        glTexCoordPointer(2, GL_FLOAT, 0, textureCoordinateBuffer.data());
    }
    if (hasColor) {
        glColorPointer(3, GL_FLOAT, 0, colorBuffer.data());
    }

    // Enable client states
    glEnableClientState(GL_VERTEX_ARRAY);
    if (hasTexture) {
        glEnableClientState(GL_TEXTURE_COORD_ARRAY);
    }
    if (hasColor) {
        glEnableClientState(GL_COLOR_ARRAY);
    }

    // Draw quads
    glDrawArrays(GL_QUADS, 0, vertices);

    // Reset after rendering
    glDisableClientState(GL_VERTEX_ARRAY);
    if (hasTexture) {
        glDisableClientState(GL_TEXTURE_COORD_ARRAY);
    }
    if (hasColor) {
        glDisableClientState(GL_COLOR_ARRAY);
    }
    clear();
    glCheckError();
}
