#ifndef TESSELATOR_H_INCLUDED
#define TESSELATOR_H_INCLUDED

#include <GL/glx.h>    /* this includes the necessary X headers */
#include <GL/gl.h>

#include <X11/X.h>    /* X11 constant (e.g. TrueColor) */
#include <X11/keysym.h>

#include <vector>
#include <string>

#include <cstdio>
#include <iostream>

static GLenum glCheckError_(const char *file, int line)
{
    GLenum errorCode;
    while ((errorCode = glGetError()) != GL_NO_ERROR)
    {
        std::string error;
        switch (errorCode)
        {
            case GL_INVALID_ENUM:                  error = "INVALID_ENUM"; break;
            case GL_INVALID_VALUE:                 error = "INVALID_VALUE"; break;
            case GL_INVALID_OPERATION:             error = "INVALID_OPERATION"; break;
            case GL_STACK_OVERFLOW:                error = "STACK_OVERFLOW"; break;
            case GL_STACK_UNDERFLOW:               error = "STACK_UNDERFLOW"; break;
            case GL_OUT_OF_MEMORY:                 error = "OUT_OF_MEMORY"; break;
            case GL_INVALID_FRAMEBUFFER_OPERATION: error = "INVALID_FRAMEBUFFER_OPERATION"; break;
        }
        std::cout << error << " | " << file << " (" << line << ")" << std::endl;
    }
    return errorCode;
}
#define glCheckError() glCheckError_(__FILE__, __LINE__)

using namespace std;

class Tessellator {

  private :
    vector<float> vertexBuffer;
    vector<float> textureCoordinateBuffer;
    vector<float> colorBuffer;

    int vertices = 0;

    // Texture
    bool hasTexture;
    float textureU;
    float textureV;

    // Color
    bool hasColor;
    float red;
    float green;
    float blue;

    /**
     * Reset the buffer
     */
  public :
    void init() {
        clear();
    }

    Tessellator() : vertexBuffer(), textureCoordinateBuffer(), colorBuffer(),
                    hasTexture(false), textureU(0.), textureV(0.), hasColor(false),
                    red(0.), green(0.), blue(0.) {};

    void vertex(float x, float y, float z);

    void texture(float textureU, float textureV) {
        hasTexture = true;
        textureU = textureU;
        textureV = textureV;
    }

    void color(float red, float green, float blue) {
        hasColor = true;
        red = red;
        green = green;
        blue = blue;
    }

    void flush();

    /**
     * Reset vertex buffer
     */
    void clear() {
        vertexBuffer.clear();
        textureCoordinateBuffer.clear();
        vertices = 0;

        hasTexture = false;
        hasColor = false;
    }
};


#endif // TESSELATOR_H_INCLUDED
