#include "Textures.h"

int Textures::lastId = ~(~0 >> 1);

    /**
     * Load a texture into OpenGL
     *
     * @param resourceName Resource path of the image
     * @param mode         Texture filter mode (GL_NEAREST, GL_LINEAR)
     * @return Texture id of OpenGL
     */

int Textures::loadTexture(const char *resourceName, int mode) {
    // Generate a new texture id
    unsigned int id;
    glGenTextures(1, &id);

    // Bind this texture id
    bind(id);

    // Set texture filter mode
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, mode);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, mode);

    // Read from resources
    ifstream file(resourceName);

    // Read to buffered image (.BMP)
    char header[54];
    file.read(header, 54);
    unsigned int dataPos    = *(int*)&(header[0x0A]);
    unsigned int width      = *(int*)&(header[0x12]);
    unsigned int height     = *(int*)&(header[0x16]);

    // Write image pixels into array
    int *pixels = new int[width * height];
    file.seekg(dataPos);
    file.read((char*)pixels, width * height * sizeof(*pixels));

    // Write texture to opengl
    gluBuild2DMipmaps(GL_TEXTURE_2D, GL_RGB, width, height, GL_RGB, GL_UNSIGNED_BYTE, pixels);

    return id;
}
