#ifndef TIMER_H_INCLUDED
#define TIMER_H_INCLUDED

#define NS_PER_SECOND 1000000000L
#define MAX_NS_PER_UPDATE 1000000000L
#define MAX_TICKS_PER_UPDATE 100

#include <ctime>
#include <algorithm>

using namespace std;

class Timer {

    private :
    /**
     * Amount of ticks per second
     */
    double ticksPerSecond;

    /**
     * Last time updated in nano seconds
     */
    clock_t lastTime;

    /**
     * Scale the tick speed
     */
    public :
    double timeScale;

    /**
     * Framerate of the advanceTime update
     */
    double fps;

    /**
     * Passed time since last game update
     */
    double passedTime;

    /**
     * The amount of ticks for the current game update.
     * It's the passed time as an integer
     */
    int ticks;

    /**
     * The overflow of the current tick, caused by casting the passed time to an integer
     */
    double partialTicks;

    /**
     * Timer to control the tick speed independently of the framerate
     *
     * @param ticksPerSecond Amount of ticks per second
     */
    Timer(double ticksPerSecond) : ticksPerSecond(ticksPerSecond), lastTime(clock()), timeScale(1.), fps(0.), passedTime(0.), ticks(0), partialTicks(0) {}

    void advanceTime();
};


#endif // TIMER_H_INCLUDED
